From 22e09ffe3afdc49ea9c77005e1a0c789f3736110 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Mon, 22 Feb 2010 17:20:27 +0100
Subject: [PATCH 6/6] input: Fill in the input_item es.

Instead of using es_format_t we use a custom structure to limit the memory footprint.
This allow libvlc_media_get_es() to properly work after libvlc_media_parse().
---
 include/vlc_input_item.h |   32 ++++++++++++++++++++++
 src/control/media.c      |   34 +++++++++++-----------
 src/input/es_out.c       |    6 +++-
 src/input/item.c         |   67 ++++++++++++++++++++++++++++++++++++++++++++++
 src/input/item.h         |    1 +
 5 files changed, 122 insertions(+), 18 deletions(-)

diff --git a/include/vlc_input_item.h b/include/vlc_input_item.h
index d4858c9..cc93557 100644
--- a/include/vlc_input_item.h
+++ b/include/vlc_input_item.h
@@ -51,6 +51,35 @@ struct info_category_t
     struct info_t **pp_infos;     /**< Pointer to an array of infos */
 };
 
+/* Stripped down version of es_format_t because es_format_t can become very
+ * large in term of memory. This is mostly libvlc_media_es_t */
+typedef struct input_item_track_t
+{
+    vlc_fourcc_t i_codec;
+    int          i_id;
+    int          i_cat; /* @see es_format_category_e */
+
+    /* Codec specific */
+    int         i_profile;
+    int         i_level;
+
+    union
+    {
+        struct
+        {
+            /* Video specific */
+            unsigned    i_height;
+            unsigned    i_width;
+        } video;
+        struct
+        {
+            /* Audio specific */
+            unsigned    i_channels;
+            unsigned    i_rate;
+        } audio;
+    } u;
+} input_item_track_t;
+
 struct input_item_t
 {
     VLC_GC_MEMBERS
@@ -73,6 +102,9 @@ struct input_item_t
     int         i_es;                /**< Number of es format descriptions */
     es_format_t **es;                /**< Es formats */
 
+    int i_tracks;                    /**< Number of track info descriptions */
+    input_item_track_t **tracks;     /**< Tracks Info */
+
     input_stats_t *p_stats;          /**< Statistics */
     int           i_nb_played;       /**< Number of times played */
 
diff --git a/src/control/media.c b/src/control/media.c
index 1e8675b..5687595 100644
--- a/src/control/media.c
+++ b/src/control/media.c
@@ -681,39 +681,39 @@ libvlc_media_get_user_data( libvlc_media_t * p_md )
  * Get media descriptor's elementary streams description
  **************************************************************************/
 int
-libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t ** pp_es )
+libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **pp_tracks )
 {
     assert( p_md );
 
     input_item_t *p_input_item = p_md->p_input_item;
     vlc_mutex_lock( &p_input_item->lock );
 
-    const int i_es = p_input_item->i_es;
-    *pp_es = (i_es > 0) ? malloc( i_es * sizeof(libvlc_media_track_info_t) ) : NULL;
+    const int i_tracks = p_input_item->i_tracks;
+    *pp_tracks = (i_tracks > 0) ? malloc( i_tracks * sizeof(*pp_tracks) ) : NULL;
 
-    if( !pp_es ) /* no ES, or OOM */
+    if( !pp_tracks ) /* no ES, or OOM */
     {
         vlc_mutex_unlock( &p_input_item->lock );
         return 0;
     }
 
     /* Fill array */
-    for( int i = 0; i < i_es; i++ )
+    for( int i = 0; i < i_tracks; i++ )
     {
-        libvlc_media_track_info_t *p_mes = *pp_es+i;
-        const es_format_t *p_es = p_input_item->es[i];
+        libvlc_media_track_info_t *p_mes = *pp_tracks+i;
+        const input_item_track_t *p_track = p_input_item->tracks[i];
 
         p_mes->i_channels = p_mes->i_rate = 0;
         p_mes->i_width = p_mes->i_height = 0;
 
 
-        p_mes->i_codec = p_es->i_codec;
-        p_mes->i_id = p_es->i_id;
+        p_mes->i_codec = p_track->i_codec;
+        p_mes->i_id = p_track->i_id;
 
-        p_mes->i_profile = p_es->i_profile;
-        p_mes->i_level = p_es->i_level;
+        p_mes->i_profile = p_track->i_profile;
+        p_mes->i_level = p_track->i_level;
 
-        switch(p_es->i_cat)
+        switch(p_track->i_cat)
         {
         case UNKNOWN_ES:
         default:
@@ -721,13 +721,13 @@ libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **
             break;
         case VIDEO_ES:
             p_mes->i_type = libvlc_track_video;
-            p_mes->i_height = p_es->video.i_height;
-            p_mes->i_width = p_es->video.i_width;
+            p_mes->i_height = p_track->u.video.i_height;
+            p_mes->i_width = p_track->u.video.i_width;
             break;
         case AUDIO_ES:
             p_mes->i_type = libvlc_track_audio;
-            p_mes->i_channels = p_es->audio.i_channels;
-            p_mes->i_rate = p_es->audio.i_rate;
+            p_mes->i_channels = p_track->u.audio.i_channels;
+            p_mes->i_rate = p_track->u.audio.i_rate;
             break;
         case SPU_ES:
             p_mes->i_type = libvlc_track_text;
@@ -736,5 +736,5 @@ libvlc_media_get_tracks_info( libvlc_media_t *p_md, libvlc_media_track_info_t **
     }
 
     vlc_mutex_unlock( &p_input_item->lock );
-    return i_es;
+    return i_tracks;
 }
diff --git a/src/input/es_out.c b/src/input/es_out.c
index 3062d8e..53ce124 100644
--- a/src/input/es_out.c
+++ b/src/input/es_out.c
@@ -45,6 +45,7 @@
 #include "es_out.h"
 #include "event.h"
 #include "info.h"
+#include "item.h"
 
 #include "../stream_output/stream_output.h"
 
@@ -2223,7 +2224,7 @@ static int EsOutControlLocked( es_out_t *out, int i_query, va_list args )
             }
             return VLC_SUCCESS;
         }
- 
+
         case ES_OUT_SET_ES_DEFAULT:
         {
             es_out_id_t *es = va_arg( args, es_out_id_t * );
@@ -2811,6 +2812,9 @@ static void EsOutUpdateInfo( es_out_t *out, es_out_id_t *es, const es_format_t *
     const es_format_t *p_fmt_es = &es->fmt;
     lldiv_t         div;
 
+    /* Update the item tracks field */
+    input_item_UpdateTrack(input_GetItem(p_input), fmt);
+
     /* Create category */
     char psz_cat[128];
     snprintf( psz_cat, sizeof(psz_cat),_("Stream %d"), es->i_meta_id );
diff --git a/src/input/item.c b/src/input/item.c
index 416ec19..61a5529 100644
--- a/src/input/item.c
+++ b/src/input/item.c
@@ -36,6 +36,7 @@
 #include "info.h"
 
 static int GuessType( const input_item_t *p_item );
+static void input_item_track_Delete(input_item_track_t *track);
 
 /** Stuff moved out of vlc_input.h -- FIXME: should probably not be inline
  * anyway. */
@@ -46,6 +47,7 @@ static inline void input_item_Init( vlc_object_t *p_o, input_item_t *p_i )
     p_i->psz_name = NULL;
     p_i->psz_uri = NULL;
     TAB_INIT( p_i->i_es, p_i->es );
+    TAB_INIT( p_i->i_tracks, p_i->tracks );
     TAB_INIT( p_i->i_options, p_i->ppsz_options );
     p_i->optflagv = NULL, p_i->optflagc = 0;
     TAB_INIT( p_i->i_categories, p_i->pp_categories );
@@ -99,6 +101,10 @@ static inline void input_item_Clean( input_item_t *p_i )
     }
     TAB_CLEAN( p_i->i_es, p_i->es );
 
+    for( i = 0; i < p_i->i_tracks; i++ )
+        input_item_track_Delete( p_i->tracks[i] );
+    TAB_CLEAN( p_i->tracks, p_i->tracks );
+
     for( i = 0; i < p_i->i_epg; i++ )
         vlc_epg_Delete( p_i->pp_epg[i] );
     TAB_CLEAN( p_i->i_epg, p_i->pp_epg );
@@ -1021,3 +1027,64 @@ void input_item_node_PostAndDelete( input_item_node_t *p_root )
 
   input_item_node_Delete( p_root );
 }
+
+/*
+ * Item tracks_info
+ */
+static input_item_track_t *input_item_track_NewFromESFormat(const es_format_t *fmt)
+{
+    input_item_track_t *ret = malloc(sizeof(*ret));
+    ret->i_codec = fmt->i_codec;
+    ret->i_id = fmt->i_id;
+    ret->i_cat = fmt->i_cat;
+
+    ret->i_profile = fmt->i_profile;
+    ret->i_level = fmt->i_level;
+
+    switch(fmt->i_cat)
+    {
+    case VIDEO_ES:
+        ret->u.video.i_height = fmt->video.i_height;
+        ret->u.video.i_width = fmt->video.i_width;
+        break;
+    case AUDIO_ES:
+        ret->u.audio.i_channels = fmt->audio.i_channels;
+        ret->u.audio.i_rate = fmt->audio.i_rate;
+        break;
+    default:
+        break;
+    }
+    return ret;
+}
+
+static void input_item_track_Delete(input_item_track_t *track)
+{
+    free(track);
+}
+
+/* Called by es_out when a new Elementary Stream is added or updated. */
+void input_item_UpdateTrack(input_item_t *item, const es_format_t *fmt)
+{
+    input_item_track_t *track = input_item_track_NewFromESFormat( fmt );
+    if( !track )
+        return;
+
+    vlc_mutex_lock( &item->lock );
+
+    for( int i = 0; i < item->i_tracks; i++ )
+    {
+        if( item->tracks[i]->i_id == track->i_id )
+        {
+            /* We've found the right ES, replace it */
+            input_item_track_Delete( item->tracks[i] );
+            item->tracks[i] = track;
+            vlc_mutex_unlock( &item->lock );
+            return;
+        }
+    }
+
+    /* ES not found, create a new one. */
+    TAB_APPEND(item->i_tracks, item->tracks, track);
+
+    vlc_mutex_unlock( &item->lock );
+}
diff --git a/src/input/item.h b/src/input/item.h
index 971620a..e09a765 100644
--- a/src/input/item.h
+++ b/src/input/item.h
@@ -31,5 +31,6 @@
 #include "input_interface.h"
 
 void input_item_SetErrorWhenReading( input_item_t *p_i, bool b_error );
+void input_item_UpdateTrack( input_item_t *item, const es_format_t *fmt );
 
 #endif
-- 
1.7.0

